use axum::extract::{Path, State};
use crate::AppState;
use askama::Template;
use axum::http::StatusCode;
use axum::response::IntoResponse;
use crate::options::{Opt, Options};

use sea_orm::*;
use tracing::log::error;
use crate::entities::{post, post_tag, tag, file};
use crate::entities::prelude::{File, Post, PostTag, Tag};
use crate::frontend::NotFoundTemplate;

#[derive(Template)]
#[template(path = "post.html")]
pub struct PostTemplate {
    site_name: String,
    url: String,
    asset_endpoint: String,
    image_endpoint: String,
    post_id: i64,
    tags: Vec<String>,
    description: String,
    hash: String,
    file_type: FileType,
    resx: i64,
    resy: i64,
    file_size: i64,
    file_id: i64,
}

pub async fn post(
    State(state): State<AppState>,
    Path(id): Path<i64>
) -> impl IntoResponse {
    let options = Options::get();

    let post: Option<post::Model> = Post::find_by_id(id)
        .one(&state.conn)
        .await
        .expect("Failed to access database?");

    if post.is_none() {
        return (StatusCode::NOT_FOUND, NotFoundTemplate {
            site_name: options.name,
            url: options.url,
            asset_endpoint: options.asset_endpoint,
        }).into_response();
    }

    let post = post.unwrap();

    let file: Option<file::Model> = File::find_by_id(post.file)
        .one(&state.conn)
        .await
        .expect("Failed to access database?");

    if file.is_none() {
        error!("Post {} exists but no files for it exist!", post.id);
        return (StatusCode::INTERNAL_SERVER_ERROR, ()).into_response();
    }

    let file = file.unwrap();

    let tags_raw: Vec<post_tag::Model> = PostTag::find()
        .filter(post_tag::Column::PostId.eq(post.id))
        .all(&state.conn)
        .await
        .expect("Failed to access database?");

    let mut tags: Vec<String> = vec![];
    for i in tags_raw {
        let tag = Tag::find_by_id(i.tag_id)
            .one(&state.conn)
            .await
            .expect("Failed to access database?");

        if tag.is_some() {
            tags.push(tag.unwrap().name);
        }
    }

    (StatusCode::OK, PostTemplate {
        site_name: options.name,
        url: options.url,
        asset_endpoint: options.asset_endpoint,
        image_endpoint: options.image_endpoint,
        post_id: post.id,
        tags,
        description: post.description,
        hash: file.hash,
        file_type: file.r#type.into(),
        resx: file.res_x,
        resy: file.res_y,
        file_size: file.size,
        file_id: file.id,
    }).into_response()
}

// TODO: Move this to uploading module when that starts being worked on
pub enum FileType {
    Image,
    Video,
    Unsupported
}

impl From<FileType> for i32 {
    fn from(code: FileType) -> i32 {
        match code {
            FileType::Image => 0,
            FileType::Video => 1,
            FileType::Unsupported => 9999999
        }
    }
}

impl From<i32> for FileType {
    fn from(code: i32) -> FileType {
        match code {
            0 => FileType::Image,
            1 => FileType::Video,
            _ => FileType::Unsupported
        }
    }
}