mod options;
mod api;
mod frontend;
mod entities;
mod rustflake;

use std::net::SocketAddr;
use axum::Router;
use migration::Migrator;
use sea_orm::{ConnectOptions, Database, DatabaseConnection};
use sea_orm_migration::MigratorTrait;
use tracing::{info, log};
use crate::api::api;
use crate::frontend::frontend;
use crate::options::{Opt, Options};

const VERSION: &str = env!("CARGO_PKG_VERSION");

#[tokio::main]
async fn main() {
    tracing_subscriber::fmt::init();

    let options = Options::get();

    info!("Starting CorrugatedCardboard v{}", VERSION);

    info!("\tConfig:");
    info!("\t\tInstance Name: {}", options.name);
    info!("\t\tListen Address: {}", options.listen_addr);
    info!("\t\tRegistrations Enabled: {}", options.registration);
    info!("\t\tURL: {}", options.url);
    info!("\t\tImages Endpoint: {}", options.image_endpoint);
    info!("\t\tAssets Endpoint: {}", options.asset_endpoint);

    // Database
    info!("Connecting to database & checking for migrations");

    let mut migration_db_opt = migration::sea_orm::ConnectOptions::new(options.database_url.clone());
    migration_db_opt.sqlx_logging_level(log::LevelFilter::Debug);

    let migrator_conn = migration::sea_orm::Database::connect(migration_db_opt)
        .await
        .expect("Failed to connect to the database!");
    Migrator::up(&migrator_conn, None)
        .await
        .expect("Failed to run migrations!");

    let mut db_opt = ConnectOptions::new(options.database_url.clone());
    db_opt.sqlx_logging_level(log::LevelFilter::Debug);

    let conn: DatabaseConnection = Database::connect(db_opt)
        .await
        .expect("Failed to connect to database!");

    info!("Connected to database");

    // Axum setup
    info!("Starting server");

    let state = AppState { conn };

    let app = Router::new()
        .merge(frontend())
        .nest("/api", api())

        .with_state(state);

    let addr: SocketAddr = options.listen_addr
        .parse()
        .expect("Unable to parse listen address!");

    info!("Listening on {}", &addr);

    axum::Server::bind(&addr)
        .serve(app.into_make_service())
        .await
        .expect("Failed to start the server!");
}

#[derive(Clone)]
pub struct AppState {
    pub conn: DatabaseConnection,
}